#ifndef std_c_h
#define std_c_h

/*---------------*/
/* Standard types*/
/*---------------*/
#ifndef u_int
typedef unsigned int        u_int   ;
#endif
#ifndef u_short
typedef unsigned short      u_short ;
#endif
#ifndef u_char
typedef unsigned char       u_char ;
#endif
/*--------------*/
/* AT91 Register*/
/*--------------*/
#define at91_reg            volatile u_int

/*---------------*/
/* Boolean values*/
/*---------------*/
#define TRUE                1
#define FALSE               0

#endif /* std_c_h */


#ifndef ebi_h
#define ebi_h

/*----------------------------------------*/
/* Memory Controller Interface Definition */
/*----------------------------------------*/

typedef struct
{
    at91_reg        EBI_CSR[8] ;        /* Chip Select Register */
    at91_reg        EBI_RCR ;           /* Remap Control Register */
    at91_reg        EBI_MCR ;           /* Memory Control Register */
} StructEBI ;

/*-----------------------*/
/* Chip Select Registers */
/*-----------------------*/

/* Data Bus Width */
#define DataBus16               (1<<0)
#define DataBus8                (2<<0)
#define DBW                     (3<<0)

/* Number of Wait States */
#define B_NWS                   2
#define WaitState1              (0<<B_NWS)
#define WaitState2              (1<<B_NWS)
#define WaitState3              (2<<B_NWS)
#define WaitState4              (3<<B_NWS)
#define WaitState5              (4<<B_NWS)
#define WaitState6              (5<<B_NWS)
#define WaitState7              (6<<B_NWS)
#define WaitState8              (7<<B_NWS)
#define NWS                     (7<<B_NWS)

/* Wait State Enable */
#define WaitStateDisable        (0<<5)
#define WaitStateEnable         (1<<5)
#define WSE                     (1<<5)

/* Page size */
#define PageSize1M              (0<<7)
#define PageSize4M              (1<<7)
#define PageSize16M             (2<<7)
#define PageSize64M             (3<<7)
#define PAGES                   (3<<7)

/* Number of Data Float Output Time Clock Cycle */
#define B_TDF                   9
#define tDF_0cycle              (0<<B_TDF)
#define tDF_1cycle              (1<<B_TDF)
#define tDF_2cycle              (2<<B_TDF)
#define tDF_3cycle              (3<<B_TDF)
#define tDF_4cycle              (4<<B_TDF)
#define tDF_5cycle              (5<<B_TDF)
#define tDF_6cycle              (6<<B_TDF)
#define tDF_7cycle              (7<<B_TDF)
#define TDF                     (7<<B_TDF)

/* Byte Access Type */
#define ByteWriteAccessType (0<<12)
#define ByteSelectAccessType    (1<<12)
#define BAT                     1<<12)

/* Chip Select Enable */
#define CSEnable                (1<<13)
#define CSDisable               (0<<13)
#define CSE                     (1<<13)

#define BA                      ((u_int)(0xFFF)<<20)

/*-------------------------*/
/* Memory Control Register */
/*-------------------------*/

/* Address Line Enable */
#define ALE                     (7<<0)
#define BankSize16M             (0<<0)
#define BankSize8M              (4<<0)
#define BankSize4M              (5<<0)
#define BankSize2M              (6<<0)
#define BankSize1M              (7<<0)

/* Data Read Protocol */
#define StandardReadProtocol    (0<<4)
#define EarlyReadProtocol       (1<<4)
#define DRP                     (1<<4)

/*------------------------*/
/* Remap Control Register */
/*------------------------*/

#define RCB                     (1<<0)

/*--------------------------------*/
/* Device Dependancies Definition */
/*--------------------------------*/


/* External Bus Interface User Interface BAse Address */
#define EBI_BASE            ((StructEBI *) 0xFFE00000)


#endif /* ebi_h */

#ifndef aic_h
#define aic_h

/*-------------------------------------------*/
/* Interrupt Controller Structure Definition */
/*-------------------------------------------*/

typedef struct
{
    at91_reg        AIC_SMR[32] ;       /* Source Mode Register */
    at91_reg        AIC_SVR[32] ;       /* Source Vector Register */
    at91_reg        AIC_IVR ;           /* Interrupt Vector Register */
    at91_reg        AIC_FVR ;           /* FIQ Vector Register */
    at91_reg        AIC_ISR ;           /* Interrupt Status Register */
    at91_reg        AIC_IPR ;           /* Interrupt Pending Register */
    at91_reg        AIC_IMR ;           /* Interrupt Mask Register */
    at91_reg        AIC_CISR ;          /* Core Interrupt Status Register */
    at91_reg        reserved0 ;
    at91_reg        reserved1 ;
    at91_reg        AIC_IECR ;          /* Interrupt Enable Command Register */
    at91_reg        AIC_IDCR ;          /* Interrupt Disable Command Register */
    at91_reg        AIC_ICCR ;          /* Interrupt Clear Command Register */
    at91_reg        AIC_ISCR ;          /* Interrupt Set Command Register */
    at91_reg        AIC_EOICR ;         /* End of Interrupt Command Register */
} StructAIC ;

/*---------------------------------*/
/* Interrupt Source Mode Registers */
/*---------------------------------*/

/* Source Type Definition */
#define LevelSensitive              (0<<5)
#define EdgeTriggered               (1<<5)
#define LowLevel                    (0<<5)
#define NegativeEdge                (1<<5)
#define HighLevel                   (2<<5)
#define PositiveEdge                (3<<5)

#define SRCTYPE                     (3<<5)
#define PRIOR                       (7<<0)

/*---------------------------*/
/* Interrupt Status Register */
/*---------------------------*/
#define IRQID                       0x1F

/*--------------------------------*/
/* Interrupt Core Status Register */
/*--------------------------------*/
#define NFIQ                        (1<<0)
#define NIRQ                        (1<<1)

/*---------------------------------------*/
/* AIC Interrupt Handler type definition */
/*---------------------------------------*/

typedef void (*TypeAICHandler) (void) ;

/*--------------------------------*/
/* Device Dependancies Definition */
/*--------------------------------*/

/* Advanced Interrupt Controller is always at same address for vectoring */
#define AIC_BASE            ((StructAIC *) 0xFFFFF000 )



/* Interrupt Sources Definition */

#define FIQ             0
#define SWIRQ           1
#define US0IRQ          2
#define US1IRQ          3
#define TC0IRQ          4
#define TC1IRQ          5
#define TC2IRQ          6

#define WDIRQ           7
#define PIOIRQ          8

#define IRQ0            16
#define IRQ1            17
#define IRQ2            18

/* Really implemented in the 40400 not including the FIQ */
#define NB_INTERRUPT    18



#endif /* aic_h */


#ifndef pio_h
#define pio_h

/*---------------------------------------------*/
/* Parallel I/O Interface Structure Definition */
/*---------------------------------------------*/

typedef struct
{
    at91_reg        PIO_PER ;           /* PIO Enable Register */
    at91_reg        PIO_PDR ;           /* PIO Disable Register */
    at91_reg        PIO_PSR ;           /* PIO Status Register */
    at91_reg        Reserved0 ;
    at91_reg        PIO_OER ;           /* Output Enable Register */
    at91_reg        PIO_ODR ;           /* Output Disable Register */
    at91_reg        PIO_OSR ;           /* Output Status Register */
    at91_reg        Reserved1 ;
    at91_reg        PIO_IFER ;          /* Input Filter Enable Register */
    at91_reg        PIO_IFDR ;          /* Input Filter Disable Register */
    at91_reg        PIO_IFSR ;          /* Input Filter Status Register */
    at91_reg        Reserved2 ;
    at91_reg        PIO_SODR ;          /* Set Output Data Register */
    at91_reg        PIO_CODR ;          /* Clear Output Data Register */
    at91_reg        PIO_ODSR ;          /* Output Data Status Register */
    at91_reg        PIO_PDSR ;          /* Pin Data Status Register */
    at91_reg        PIO_IER ;           /* Interrupt Enable Register */
    at91_reg        PIO_IDR ;           /* Interrupt Disable Register */
    at91_reg        PIO_IMR ;           /* Interrupt Mask Register */
    at91_reg        PIO_ISR ;           /* Interrupt Status Register */
} StructPIO ;

/*-----------------------------*/
/* PIO Handler type definition */
/*-----------------------------*/

typedef void (*TypePIOHandler) ( StructPIO *pio_pt, u_int pio_mask ) ;

/*--------------------------------*/
/* Device Dependancies Definition */
/*--------------------------------*/


/* Number of PIO Controller */
#define NB_PIO_CTRL     1
/* Base Address */
#define PIO_BASE        ((StructPIO *) 0xFFFF0000 )
/* Number of PIO Lines */
#define NB_PIO          32

/* Parallel I/O Bits Definition */
#define P0              (1<<0)
#define P1              (1<<1)
#define P2              (1<<2)
#define P3              (1<<3)
#define P4              (1<<4)
#define P5              (1<<5)
#define P6              (1<<6)
#define P7              (1<<7)
#define P8              (1<<8)
#define P9              (1<<9)
#define P10             (1<<10)
#define P11             (1<<11)
#define P12             (1<<12)
#define P13             (1<<13)
#define P14             (1<<14)
#define P15             (1<<15)
#define P16             (1<<16)
#define P17             (1<<17)
#define P18             (1<<18)
#define P19             (1<<19)
#define P20             (1<<20)
#define P21             (1<<21)
#define P22             (1<<22)
#define P23             (1<<23)
#define P24             (1<<24)
#define P25             (1<<25)
#define P26             (1<<26)
#define P27             (1<<27)
#define P28             (1<<28)
#define P29             (1<<29)
#define P30             (1<<30)
#define P31             (1<<31)

/* PIO Multiplexing Definition */

/* There is only one PIO Controller */
#define PIO_CTRL        0

#define PIO_TC0         PIO_CTRL
#define TCLK0           P0
#define TIOA0           P1
#define TIOB0           P2
#define PIN_TC0         (TIOA0|TIOB0|TCLK0)

#define PIO_TC1         PIO_CTRL
#define TCLK1           P3
#define TIOA1           P4
#define TIOB1           P5
#define PIN_TC1         (TIOA1|TIOB1|TCLK1)

#define PIO_TC2         PIO_CTRL
#define TCLK2           P6
#define TIOA2           P7
#define TIOB2           P8
#define PIN_TC2         (TIOA2|TIOB2|TCLK2)

#define PIO_EXT_IRQ     PIO_CTRL
#define PIN_IRQ0        P9
#define PIN_IRQ1        P10
#define PIN_IRQ2        P11
#define PIN_FIQ         P12

#define PIO_USART0      PIO_CTRL
#define SCK0            P13
#define TXD0            P14
#define RXD0            P15
#define PIN_USART0      (SCK0|TXD0|RXD0)

#define PIO_USART1      PIO_CTRL
#define SCK1            P20
#define TXD1            P21
#define RXD1            P22
#define PIN_USART1      (SCK1|TXD1|RXD1)

#define MCKO            P25
#define CS2             P26
#define CS3             P27
#define CS4             P31
#define CS5             P30
#define CS6             P29
#define CS7             P28



#endif /* pio_h */
#ifndef usart_h
#define usart_h

/*----------------------------*/
/* USART Structure Definition */
/*----------------------------*/

typedef struct
{
    at91_reg        US_CR ;         /* Control Register */
    at91_reg        US_MR ;         /* Mode Register */
    at91_reg        US_IER ;        /* Interrupt Enable Register */
    at91_reg        US_IDR ;        /* Interrupt Disable Register */
    at91_reg        US_IMR ;        /* Interrupt Mask Register */
    at91_reg        US_CSR ;        /* Channel Status Register */
    at91_reg        US_RHR ;        /* Receive Holding Register */
    at91_reg        US_THR ;        /* Transmit Holding Register */
    at91_reg        US_BRGR ;       /* Baud Rate Generator Register */
    at91_reg        US_RTOR ;       /* Receiver Timeout Register */
    at91_reg        US_TTGR ;       /* Transmitter Timeguard Register */
    at91_reg        Reserved ;
    at91_reg        US_RPR ;        /* Receiver Pointer Register */
    at91_reg        US_RCR ;        /* Receiver Counter Register */
    at91_reg        US_TPR ;        /* Transmitter Pointer Register */
    at91_reg        US_TCR ;        /* Transmitter Counter Register */
} StructUSART ;

/*------------------*/
/* Control Register */
/*------------------*/
/* Reset Receiver */
#define RSTRX                   (1<<2)
/* Reset Transmitter */
#define RSTTX                   (1<<3)
/* Receiver Enable */
#define RXEN                    (1<<4)
/* Receiver Disable */
#define RXDIS                   (1<<5)
/* Transmitter Enable */
#define TXEN                    (1<<6)
/* Transmitter Disable */
#define TXDIS                   (1<<7)
/* Reset Status Bits */
#define RSTSTA                  (1<<8)
/* Start Break */
#define STTBRK                  (1<<9)
/* Stop Break */
#define STPBRK                  (1<<10)
/* Start Timeout */
#define STTTO                   (1<<11)
/* Send Address */
#define SENDA                   (1<<12)
/* Force Load */
#define FORCEL                  (1<<15)

/*---------------*/
/* Mode Register */
/*---------------*/

/* Clock Selection */
#define CLKS                    (3<<4)
#define USClk_MCK               (0<<4)
#define USClk_MCK8              (1<<4)
#define USClk_SCK               (2<<4)

/* Byte Length */
#define CHRL                    (3<<6)
#define FiveBits                (0<<6)
#define SixBits                 (1<<6)
#define SevenBits               (2<<6)
#define EightBits               (3<<6)

/* Synchronous Mode Enable */
#define SYNC                    (1<<8)

/* Parity Mode */
#define PAR                     (7<<9)
#define EvenParity              (0<<9)
#define OddParity               (1<<9)
#define SpaceParity             (2<<9)
#define MarkParity              (3<<9)
#define NoParity                (4<<9)
#define MultiDropMode           (6<<9)

/* Stop Bit Number */
#define NBSTOP                  (3<<12)
#define Stop1Bit                (0<<12)
#define Stop1_5Bit              (1<<12)
#define Stop2Bit                (2<<12)

/* Channel Mode */
#define CHMODE                  (3<<14)
#define NormalMode              (0<<14)
#define AutomaticEcho           (1<<14)
#define LocalLoopback           (2<<14)
#define RemoteLoopback          (3<<14)

/* 9 Bit Mode */
#define MODE9                   (1<<17)

/* Baud Rate Output Enable */
#define CLKO                    (1<<18)

/* Standard Asynchronous Mode : 8 bits , 1 stop , no parity */
#define StandardAsyncMode   ( NormalMode + \
                              Stop1Bit + \
                              NoParity + \
                              EightBits + \
                              USClk_MCK )

/* Standard External Asynchronous Mode : 8 bits , 1 stop , no parity */
#define ExternalStandardAsyncMode   ( NormalMode + \
                                      Stop1Bit + \
                                      NoParity + \
                                      EightBits + \
                                      USClk_SCK )

/* Standard Synchronous Mode : 8 bits , 1 stop , no parity */
#define StandardSyncMode    ( SYNC + \
                              NormalMode + \
                              Stop1Bit + \
                              NoParity + \
                              EightBits + \
                              USClk_MCK )

/* SCK used Label */
#define SCK_USED                (CLKO | USClk_SCK)

/*-------------------------*/
/* Channel Status Register */
/*-------------------------*/

/* Receiver Ready */
#define RXRDY                   (1<<0)

/* Transmitter Ready */
#define TXRDY                   (1<<1)

/* Receiver Break */
#define RXBRK                   (1<<2)

/* End of Receiver PDC Transfer */
#define ENDRX                   (1<<3)

/* End of Transmitter PDC Transfer */
#define ENDTX                   (1<<4)

/* Overrun Error */
#define OVRE                    (1<<5)

/* Framing Error */
#define FRAME                   (1<<6)

/* Parity Error */
#define PARE                    (1<<7)

/* Receiver Timeout */
#define TIMEOUT                 (1<<8)

/* Transmitter Empty */
#define TXEMPTY                 (1<<9)

#define MASK_IRQ_TX             (TXRDY | ENDTX | TXEMPTY)
#define MASK_IRQ_RX             (RXRDY | ENDRX | TIMEOUT)
#define MASK_IRQ_ERROR          (PARE | FRAME | OVRE | RXBRK)

/*-------------------------------*/
/* USART Handler type definition */
/*-------------------------------*/

typedef void (*TypeUSARTHandler) ( StructUSART *usart_pt ) ;

/*-------------------------------*/
/* USART Handler Table Structure */
/*-------------------------------*/

typedef struct
{
    u_int                   ErrorMask ;
    TypeUSARTHandler        ErrorHandler ;
    u_int                   RxMask ;
    TypeUSARTHandler        RxHandler ;
    u_int                   TxMask ;
    TypeUSARTHandler        TxHandler ;
} StructUSARTHandlerTable ;

/*--------------------------------*/
/* Device Dependancies Definition */
/*--------------------------------*/



#define NB_USART            2
#define USART0_BASE     ((StructUSART *) 0xFFFD0000)
#define USART1_BASE     ((StructUSART *) 0xFFFCC000)



#endif /* usart_h */
#ifndef tc_h
#define tc_h

/*----------------------------*/
/* Timer Structure Definition */
/*----------------------------*/

typedef struct
{
    at91_reg        TC_CCR ;        /* Control Register */
    at91_reg        TC_CMR ;        /* Mode Register */
    at91_reg        Reserved0 ;
    at91_reg        Reserved1 ;
    at91_reg        TC_CV ;         /* Counter value */
    at91_reg        TC_RA ;         /* Register A */
    at91_reg        TC_RB ;         /* Register B */
    at91_reg        TC_RC ;         /* Register C */
    at91_reg        TC_SR ;         /* Status Register */
    at91_reg        TC_IER ;        /* Interrupt Enable Register */
    at91_reg        TC_IDR ;        /* Interrupt Disable Register */
    at91_reg        TC_IMR ;        /* Interrupt Mask Register */
    at91_reg        Reserved2 ;
    at91_reg        Reserved3 ;
    at91_reg        Reserved4 ;
    at91_reg        Reserved5 ;
} StructTC ;

#define NB_TC_CHANNEL       3

typedef struct
{
    StructTC        TC[NB_TC_CHANNEL] ;
    at91_reg        TC_BCR ;        /* Block Control Register */
    at91_reg        TC_BMR ;        /* Block Mode Register  */
} StructTCBlock ;

/*------------------------------------------------*/
/* Timer Counter Control Register Bits Definition */
/*------------------------------------------------*/
#define CLKEN           (1<<0)
#define CLKDIS          (1<<1)
#define SWTRG           (1<<2)

/*---------------------------------------------*/
/* Timer Counter Mode Register Bits Definition */
/*---------------------------------------------*/
/* Edges Definition */
#define EdgeNone        ((u_int)0)
#define RisingEdge      ((u_int)1)
#define FallingEdge     ((u_int)2)
#define BothEdge        ((u_int)3)

/* Output Control Definition */
#define OutputNone      ((u_int)0)
#define SetOutput       ((u_int)1)
#define ClearOutput     ((u_int)2)
#define ToggleOutput    ((u_int)3)

/* Clock Selection */
#define TCCLKS          (7<<0)
#define TCMCK2          (0<<0)
#define TCMCK8          (1<<0)
#define TCMCK32         (2<<0)
#define TCMCK128        (3<<0)
#define TCMCK1024       (4<<0)
#define TCXc0           (5<<0)
#define TCXc1           (6<<0)
#define TCXc2           (7<<0)

/* Clock Inversion */
#define CLKI            (1<<3)

/* Burst Signal Selection */
#define BURST           (3<<4)
#define BurstNone       (0<<4)
#define BurstXc0        (1<<4)
#define BurstXc1        (2<<4)
#define BurstXc2        (3<<4)

/* Capture Mode : Counter Clock Stopped with RB Loading */
#define LDBSTOP         (1<<6)
/* Waveform Mode : Counter Clock Stopped with RC Compare */
#define CPCSTOP         (1<<6)

/* Capture Mode : Counter Clock Disabled with RB Loading */
#define LDBDIS          (1<<7)
/* Waveform Mode : Counter Clock Disabled with RC Compare */
#define CPCDIS          (1<<7)

/* Capture Mode : External Trigger Edge Selection */
#define B_ETRGEDG       8
#define ETRGEDG         (3<<B_ETRGEDG)
/* Waveform Mode : External Event Edge Selection */
#define B_EEVTEDG       8
#define EEVTEDG         (3<<B_EEVTEDG)

/* Capture Mode : TIOA or TIOB External Trigger Selection */
#define ABETRG          (1<<10)
/* Waveform Mode : External Event Selection */
#define B_EEVT          10
#define EEVT            (3<<B_EEVT)
#define EEVTTiob        (0<<B_EEVT)
#define EEVTXc0         (1<<B_EEVT)
#define EEVTXc1         (2<<B_EEVT)
#define EEVTXc2         (3<<B_EEVT)

/* Waveform Mode : Enable Trigger on External Event */
#define ENETRG          (1<<12)

/* RC Compare Enable Trigger Enable */
#define CPCTRG          (1<<14)

/* Mode Selection */
#define WAVE            (1<<15)
#define CAPT            (0<<15)

/* Capture Mode : RA Loading Selection */
#define B_LDRA          16
#define LDRA            (3<<B_LDRA)
/* Waveform Mode : RA Compare Effect on TIOA */
#define B_ACPA          16
#define ACPA            (3<<B_ACPA)

/* Capture Mode : RB Loading Selection */
#define B_LDRB          18
#define LDRB            (3<<B_LDRB)
/* Waveform Mode : RC Compare Effect on TIOA */
#define B_ACPC          18
#define ACPC            (3<<B_ACPC)

/* Waveform Mode : External Event Effect on TIOA */
#define B_AEEVT         20
#define AEEVT           (3<<B_AEEVT)

/* Waveform Mode : Software Trigger Effect on TIOA */
#define B_ASWTRG        22
#define ASWTRG          (3<<B_ASWTRG)

/* Waveform Mode : RB Compare Effect on TIOB */
#define B_BCPB          24
#define BCPB            (3<<B_BCPB)

/* Waveform Mode : RC Compare Effect on TIOB */
#define B_BCPC          26
#define BCPC            (3<<B_BCPC)

/* Waveform Mode : External Event Effect on TIOB */
#define B_BEEVT         28
#define BEEVT           (3<<B_BEEVT)

/* Waveform Mode : Software Trigger Effect on TIOB */
#define B_BSWTRG        30
#define BSWTRG          (((u_int)3)<<B_BSWTRG)

/*-----------------------------------------------*/
/* Timer Counter Status Register Bits Definition */
/*-----------------------------------------------*/
/* Counter Overflow Status */
#define COVFS           (1<<0)
/* Load Overrun Status */
#define LOVRS           (1<<1)
/* RA Compare Status */
#define CPAS            (1<<2)
/* RB Compare Status */
#define CPBS            (1<<3)
/* RC Compare Status */
#define CPCS            (1<<4)
/* RA Loading Status */
#define LDRAS           (1<<5)
/* RB Loading Status */
#define LDRBS           (1<<6)
/* External Trigger Status */
#define ETRGS           (1<<7)
/* Clock Status */
#define CLKSTA          (1<<16)
/* TIOA Mirror */
#define MTIOA           (1<<17)
/* TIOB Status */
#define MTIOB           (1<<18)

/*------------------------------------------------------*/
/* Timer Counter Block Control Register Bits Definition */
/*------------------------------------------------------*/
/* Synchronisation Trigger */
#define TCSYNC          1

/*---------------------------------------------------*/
/* Timer Counter Block Mode Register Bits Definition */
/*---------------------------------------------------*/
#define B_TC0XC0S       0
#define TC0XC0S         (3<<B_TC0XC0S)

#define TCLK0XC0        (0<<B_TC0XC0S)
#define NONEXC0         (1<<B_TC0XC0S)
#define TIOA1XC0        (2<<B_TC0XC0S)
#define TIOA2XC0        (3<<B_TC0XC0S)

#define B_TC1XC1S       2
#define TC1XC1S         (3<<B_TC1XC1S)

#define TCLK1XC1        (0<<B_TC1XC1S)
#define NONEXC1         (1<<B_TC1XC1S)
#define TIOA0XC1        (2<<B_TC1XC1S)
#define TIOA2XC1        (3<<B_TC1XC1S)

#define B_TC2XC2S       4
#define TC2XC2S         (3<<B_TC2XC2S)

#define TCLK2XC2        (0<<B_TC2XC2S)
#define NONEXC2         (1<<B_TC2XC2S)
#define TIOA0XC2        (2<<B_TC2XC2S)
#define TIOA1XC2        (3<<B_TC2XC2S)

/*---------------------------------------*/
/* Timer Counter Handler type definition */
/*---------------------------------------*/

typedef void (*TypeTCHandler) ( StructTC *tc_pt ) ;

/*--------------------------------*/
/* Device Dependancies Definition */
/*--------------------------------*/



#define NB_TC_BLOCK     1
#define NB_TIMER        (NB_TC_BLOCK*NB_TC_CHANNEL)
#define TCB0_BASE       ((StructTCBlock *)0xFFFE0000)



#endif /* tc_h */
#ifndef wd_h
#define wd_h

/*------------------------------------------*/
/* Watch Dog Interface Structure Definition */
/*------------------------------------------*/
typedef struct
{
    at91_reg        WD_OMR ;        /* Overflow Mode Register */
    at91_reg        WD_CMR ;        /* Clock Mode Register */
    at91_reg        WD_CR ;         /* Control Register */
    at91_reg        WD_SR ;         /* Status Register */
    at91_reg        Reserved ;
    at91_reg        WD_TLR ;        /* Test Load Register : test purpose only */
} StructWD ;

/*--------------------------------------------------*/
/* Watch Dog Overflow Mode Register Bits Definition */
/*--------------------------------------------------*/

/* Watch Dog Enable */
#define WDEN            (1<<0)

/* Internal Reset Enable */
#define RSTEN           (1<<1)

/* Interrupt Enable */
#define IRQEN           (1<<2)

/* External Signal Enable */
#define EXTEN           (1<<3)

/* Overflow Mode Register Access Key */
#define OKEY            (0x234<<4)

/*------------------------------------------*/
/* Watch Dog Clock Register Bits Definition */
/*------------------------------------------*/

/* Clock Selection */
#define WDCLKS          (3<<0)
#define WDClk8          (0<<0)
#define WDClk32         (1<<0)
#define WDClk128        (2<<0)
#define WDClk1024       (3<<0)

/* Hicg Preload Counter Value */
#define HPCV            (0xF<<2)

/* Clock Register Access Key */
#define CKEY            (0x06E<<7)

/*--------------------------------------------*/
/* Watch Dog Control Register Bits Definition */
/*--------------------------------------------*/

/* Watch Dog Restart Key */
#define RSTKEY          0xC071

/*-------------------------------------------*/
/* Watch Dog Status Register Bits Definition */
/*-------------------------------------------*/

/* WatchDog Overflow Status */
#define WDOVF           (1<<0)

/*-----------------------------------*/
/* Watch Dog Handler type definition */
/*-----------------------------------*/

typedef void (*TypeWDHandler) ( void ) ;

/*--------------------------------*/
/* Device Dependancies Definition */
/*--------------------------------*/



#define WD_BASE         (( StructWD *) 0xFFFF8000 )



#endif /* wd_h */

#ifndef ps_h
#define ps_h

/*-----------------------------------*/
/* Power Saving Structure Definition */
/*-----------------------------------*/

typedef struct
{
    at91_reg    PS_CR ; /* Control Register */
} StructPS ;

/*-----------------------------------------------*/
/* Power Saving Control Register Bits Definition */
/*-----------------------------------------------*/

#define ARM7DIS         (1<<0)

/*--------------------------------*/
/* Device Dependancies Definition */
/*--------------------------------*/



#define PS_BASE         ((StructPS *) 0xFFFF4000 )



#endif /* ps_h */
#ifndef sf_h
#define sf_h

/*-------------------------------------------------*/
/* Chip Identification Module Structure Definition */
/*-------------------------------------------------*/

typedef struct
{
    at91_reg    SF_CIDR ;       /* Chip Identification Register */
    at91_reg    SF_EXID ;       /* Chip Identification Extension */
    at91_reg    SF_RSF ;        /* Reset Status Flag */
} StructSF ;

/*----------------------------------------------*/
/* Chip Identification Register Bits Definition */
/*----------------------------------------------*/

/* Version Number */
#define VERSION         (0x1F<<0)

/* Non Volatile Program Memory Size */
#define NVPSIZ          (0x0F<<8)

/* Non Volatile Data Memory Size */
#define NVDSIZ          (0x0F<<12)

/* Volatile Data Memory Size */
#define VDSIZ           (0x0F<<16)

/* Architecture Code */
#define ARCH            (0xFF<<20)

/* Non Volatile Program Memory Type */
#define NVPTYP          (0x07<<28)

/* Extension Flag */
#define EXT             ((u_int)0x01<<31)

/*------------------------------*/
/* Reset Status Flag Definition */
/*------------------------------*/

#define EXT_RESET       0x6C
#define WD_RESET        0x53

/*--------------------------------*/
/* Device Dependancies Definition */
/*--------------------------------*/



#define SF_BASE         ((StructSF *) 0xFFF00000 )



#endif /* sf_h */

#ifdef AT91_PTRS

StructEBI	*at91_ebi	= EBI_BASE;
StructAIC	*at91_aic	= AIC_BASE;
StructPIO	*at91_pio	= PIO_BASE;
StructUSART	*at91_uart0	= USART0_BASE;
StructUSART	*at91_uart1	= USART1_BASE;
StructTCBlock	*at91_tcblock	= TCB0_BASE;
StructWD	*at91_wdog	= WD_BASE;
StructPS	*at91_ps	= PS_BASE;
StructSF	*at91_sf	= SF_BASE;

#endif	/* AT91_PTRS */
