/* tcp.h - TCPHLEN, SEQCMP */

typedef	long	tcpseq;

/*
 * SEQCMP - sequence space comparator
 *	this handles sequence space wrap-around. Overflow/Underflow makes
 * the result below correct (-,0,+) for any a,b in the sequence
 * space. Results:	results	implies
 *			  -	  a < b
 *			  0	  a = b
 *			  +	  a > b
 */

#define SEQCMP(a,b)	((a) - (b))

/* tcp packet format */

struct tcp
{
	unsigned short	tcp_sport;	/* source port */
	unsigned short	tcp_dport;	/* destination port */
	tcpseq	tcp_seq;			/* sequence */
	tcpseq	tcp_ack;			/* acknowledge sequence */
	char	tcp_offset;
	char	tcp_code;		/* control flags */
	unsigned short	tcp_window;	/* window advertisment */
	unsigned short	tcp_cksum;	/* check sum */
	unsigned short	tcp_urgptr;	/* urgent pointer */
	char	tcp_data[1];
};

/* TCP control bits */

#define TCPF_URG	0x20			/* urgent pointer is valid */
#define TCPF_ACK	0x10			/* acknowledgment field is valid */
#define TCPF_PSH	0x08			/* this segment requests a push */
#define TCPF_RST	0x04			/* reset the connection */
#define TCPF_SYN	0x02			/* synchronize sequence numbers */
#define TCPF_FIN	0x01			/* sender has reached end of its stream */

#define TCPMHLEN	20			/* minimum TCP header length */
#define TCPHOFFSET 0x50			/* tcp_offset value for TCPMHLEN */
#define TCP_HLEN(ptcp)	(((ptcp)->tcp_offset & 0xf0) >> 2)

/* TCP options */

#define TPO_EOOL	0			/* end of option list */
#define TPO_NOOP	1			/* no operation */
#define TPO_MSS	2			/* maximum segment size */

/* TCP states */

#define TCPS_FREE		0
#define TCPS_CLOSED	1
#define TCPS_LISTEN	2
#define TCPS_SYNSENT	3
#define TCPS_SYNRCVD	4
#define TCPS_ESTABLISHED	5
#define TCPS_FINWAIT1	6
#define TCPS_FINWAIT2	7
#define TCPS_CLOSEWAIT	8
#define TCPS_LASTACK	9
#define TCPS_CLOSING	10
#define TCPS_TIMEWAIT	11

#define NTCPSTATES	12

/* output states */

#define TCPO_IDLE		0
#define TCPO_PERSIST	1
#define TCPO_XMIT		2
#define TCPO_REXMT	3

#define NTCPOSTATES	4



extern int TcpInSegs;
extern int tcps_mutex;
extern int TcpEstabResets;
extern int TcpCurrEstab;
extern int TopAttemptFails;
extern int TopCurrEstab;
extern int TopPassiveOpens;
extern int TcpOutSegs;
extern int TcpRetansSegs;
extern int TcpActiveOpens;


