
#define AT91_PTRS	1
#include "../at91m40400.h"
#include "cs8900.h"

#include "ip.h"
#include "deqna.h"
#include "ether.h"
#include "arp.h"
#include "icmp.h"
#include "tcp.h"

#define TRUE	1
#define FALSE	0

#define CS8900_IO_BASE (0x2000L + (0x300L << 1))
#define IO_BASE 0x3000000L
#define MAX_IO 0x4000000L
#define IO_BASE_OFFSET (0x100000L >> 1)
/* Function prototypes */
void write_reg(unsigned short *ptr, unsigned short reg_no,unsigned short val);
unsigned short read_reg(unsigned short *ptr,unsigned short reg_no);
void show_registers(unsigned short * ptr);
void init_cs8900(unsigned short * ptr);
void poll_cs8900(unsigned short *cs8900);
void rx_event(unsigned short *cs8900);
void tx_event(unsigned short *cs8900,unsigned short ev);
void buf_event(unsigned short *cs8900, unsigned short ev);
int netin(unsigned short *cs8900,register struct epacket *packet,register int len);
int arp_in(unsigned short *cs8900,register struct epacket *packet,int len);
int ip_in(unsigned short *cs8900,register struct epacket *packet,int len);
int icmp_in(unsigned short *cs8900,register struct epacket *packet);
void tx_packet(unsigned short *cs8900,unsigned short *pkt,int len);

unsigned short  hs2net(unsigned short i);       /* host to network short */
unsigned short  net2hs(unsigned short i);       /* network to host short */
long    hl2net(long i);       /* host to network long */
long    net2hl(long i);       /* network to host long */
unsigned short cksum(register unsigned char buf[],register int nwords);
void memcpy(unsigned char *dest,unsigned char *src,int len);
int memequ(unsigned char *dest,unsigned char *src,int len);


void putlong(unsigned long val);
void putword(unsigned short val);
void puthex(unsigned char ch);
void putline(char *line);
void putc(char ch);

#define isodd(x) (01 & (int)(x))


unsigned short rx_packet[2048];

int main (void)
{
	unsigned short *ptr,val;
	
	/* intialise the serial port*/
	at91_uart0->US_CR = RSTRX | RSTTX | RSTSTA;
	at91_uart0->US_CR = TXEN;
	at91_uart0->US_MR = USClk_MCK | EightBits | NoParity | Stop2Bit | NormalMode;
	at91_uart0->US_BRGR = 32; 	/* 38400 baud  20MHz/38400 * 16 */
	putline("\r\n\nAPS-01 Network Test Program\r\n");

	ptr = (unsigned short *)(IO_BASE + CS8900_IO_BASE);

	do
	{
		val = read_reg(ptr,PP_ChipID);
		ptr += IO_BASE_OFFSET;
	}while((val != PP_ChipID_CL) && (ptr < (unsigned short *)MAX_IO));
	if (val != PP_ChipID_CL)
	{
		putline("Cannot find CS8900A network chip\r\n");
		while(1==1);
	}
	ptr -= IO_BASE_OFFSET;
		
	putline("Found CS8900A at address 0x");
	putlong((unsigned long)ptr);
	putline(" with chip id ");
	putword(val);
	putline("\r\n");
	/* switch to memory mode */
	write_reg(ptr,PP_BusCtl,PP_BusCtl_MemoryE);
	ptr -= CS8900_IO_BASE / 2;
	show_registers(ptr);
	init_cs8900(ptr);
	show_registers(ptr);
	while(1==1)
		poll_cs8900(ptr);
}

void write_reg(unsigned short *ptr, unsigned short reg_no,unsigned short val)
{
	*(ptr + CS8900A_PPTR) = reg_no;
	*(ptr + CS8900A_PDATA) = val;
}

unsigned short read_reg(unsigned short *ptr,unsigned short reg_no)
{
	unsigned short *t;
	
	t = ptr + CS8900A_PPTR;
	*(ptr + CS8900A_PPTR) = reg_no;
	t = ptr + CS8900A_PDATA;
	return(*(ptr + CS8900A_PDATA));
	
}


void __gccmain()
{
}



void putlong(unsigned long val)
{
	puthex((val >> 24) & 0xff);
	puthex((val >> 16) & 0xff);
	puthex((val >> 8) & 0xff);
	puthex(val & 0xff);
}

void putword(unsigned short val)
{
	puthex((val >> 8) & 0xff);
	puthex(val & 0xff);
}

void puthex(unsigned char ch)
{
	static char out[] ={'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'};
	
	putc(out[ch >> 4]);
	putc(out[ch & 0x0f]);
}

void putline(char *line)
{
	while(*line != '\0')
		putc(*line++);
}

void putc(char ch)
{
	while ((at91_uart0->US_CSR & TXRDY) == 0);
	at91_uart0->US_THR = ch;
}

void show_registers(unsigned short * ptr)
{
	unsigned short val;
	int i;
	unsigned  short *t;
	
	val = *(ptr + PP_IntReg);
	putline("Interrupt register 0x");
	putword(PP_IntReg);
	putline(" Register ");
	putword(val & 0x3f);
	putline(" Value 0x");
	putword(val);
	putline("\r\n");
	switch (val & 0xf)
	{
		case PP_IntReg_IRQ0 :   putline("\tINTR0 pin\r\n");
								break;
		case PP_IntReg_IRQ1 :   putline("\tINTR1 pin\r\n");
								break;
		case PP_IntReg_IRQ2 :   putline("\tINTR2 pin\r\n");
								break;
		case PP_IntReg_IRQ3 :	putline("\tINTR3 pin\r\n");
								break;
	}	
	putline("\r\n");
	
	val = *(ptr + PP_RxCFG);
	putline("Receiver configuration 0x");
	putword(PP_RxCFG);
	putline(" Register ");
	putword(val & 0x3f);
	putline(" Value 0x");
	putword(val);
	putline("\r\n");
	if ((val & PP_RxCFG_Skip1) != 0)
		putline("\tSkip (i.e. discard) current frame\r\n");
	if ((val & PP_RxCFG_Stream) != 0)
	   putline("\tEnable streaming mode\r\n");
	if ((val & PP_RxCFG_RxOK) != 0)
	    putline("\tRxOK interrupt enable\r\n");
	if ((val & PP_RxCFG_RxDMAonly) != 0)
	   putline("\tUse RxDMA for all frames\r\n");
	if ((val & PP_RxCFG_AutoRxDMA) != 0)
	   putline("\tSelect RxDMA automatically\r\n");
	if ((val & PP_RxCFG_BufferCRC) != 0)
	   putline("\tInclude CRC characters in frame\r\n");
	if ((val & PP_RxCFG_CRC) != 0)
		putline("\tEnable interrupt on CRC error\r\n");
	if ((val & PP_RxCFG_RUNT) != 0)
	   putline("\tEnable interrupt on RUNT frames\r\n");
	if ((val & PP_RxCFG_EXTRA) != 0)
	    putline("\tEnable interrupt on frames with extra data\r\n");

	val = *(ptr + PP_RxCTL);
	putline("Receiver control 0x");
	putword(PP_RxCTL);
	putline(" Register ");
	putword(val & 0x3f);
	putline(" Value 0x");
	putword(val);
	putline("\r\n");
	if ((val &  PP_RxCTL_IAHash) != 0)
	   putline("\tAccept frames that match hash\r\n");
	if ((val &  PP_RxCTL_Promiscuous) != 0)
	   putline("\tAccept any frame\r\n");
	if ((val &  PP_RxCTL_RxOK) != 0)
	   putline("\tAccept well formed frames\r\n");
	if ((val &  PP_RxCTL_Multicast) != 0)
	   putline("\tAccept multicast frames\r\n");
	if ((val &  PP_RxCTL_IA) != 0)
	   putline("\tAccept frame that matches IA\r\n");
	if ((val &  PP_RxCTL_Broadcast) != 0)
	   putline("\tAccept broadcast frames\r\n");
	if ((val &  PP_RxCTL_CRC) != 0)
	   putline("\tAccept frames with bad CRC\r\n");
	if ((val &  PP_RxCTL_RUNT) != 0)
	   putline("\tAccept runt frames\r\n");
	if ((val &  PP_RxCTL_EXTRA) != 0)
	   putline("\tAccept frames that are too long\r\n");
	   
	val = *(ptr + PP_TxCFG);
	putline("Transmit configuration 0x");
	putword(PP_TxCFG);
	putline(" Register ");
	putword(val & 0x3f);
	putline(" Value 0x");
	putword(val);
	putline("\r\n");
	if ((val & PP_TxCFG_CRS) != 0)
	   putline("\tEnable interrupt on loss of carrier\r\n");
	if ((val & PP_TxCFG_SQE) != 0)
	   putline("\tEnable interrupt on Signal Quality Error\r\n");
	if ((val & PP_TxCFG_TxOK) != 0)
	   putline("\tEnable interrupt on successful xmits\r\n");
	if ((val & PP_TxCFG_Late) != 0)
	   putline("\tEnable interrupt on \"out of window\" \r\n");
	if ((val & PP_TxCFG_Jabber) != 0)
	   putline("\tEnable interrupt on jabber detect\r\n");
	if ((val & PP_TxCFG_Collision) != 0)
	   putline("\tEnable interrupt if collision\r\n");
	if ((val & PP_TxCFG_16Collisions) != 0)
	   putline("\tEnable interrupt if > 16 collisions\r\n");

	val = *(ptr + PP_TxCmd);
	putline("Transmit command status 0x");
	putword(PP_TxCmd);
	putline(" Register ");
	putword(val & 0x3f);
	putline(" Value 0x");
	putword(val);
	putline("\r\n");
	if ((val & PP_TxCmd_TxStart_5) != 0)
	   putline("\tStart after 5 bytes in buffer\r\n");
	if ((val & PP_TxCmd_TxStart_381) != 0)
	   putline("\tStart after 381 bytes in buffer\r\n");
	if ((val & PP_TxCmd_TxStart_1021) != 0)
	   putline("\tStart after 1021 bytes in buffer\r\n");
	if ((val & PP_TxCmd_TxStart_Full) != 0)
	   putline("\tStart after all bytes loaded\r\n");
	if ((val & PP_TxCmd_Force) != 0)
	   putline("\tDiscard any pending packets\r\n");
	if ((val & PP_TxCmd_OneCollision) != 0)
	   putline("\tAbort after a single collision\r\n");
	if ((val & PP_TxCmd_NoCRC) != 0)
	   putline("\tDo not add CRC\r\n");
	if ((val & PP_TxCmd_NoPad) != 0)
	   putline("\tDo not pad short packets\r\n");

	val = *(ptr + PP_BufCFG);
	putline("Buffer configuration 0x");
	putword(PP_BufCFG);
	putline(" Register ");
	putword(val & 0x3f);
	putline(" Value 0x");
	putword(val);
	putline("\r\n");
	if ((val & PP_BufCFG_SWI) != 0)
	   putline("\tForce interrupt via software\r\n");
	if ((val & PP_BufCFG_RxDMA) != 0)
	   putline("\tEnable interrupt on Rx DMA\r\n");
	if ((val & PP_BufCFG_TxRDY) != 0)
	   putline("\tEnable interrupt when ready for Tx\r\n");
	if ((val & PP_BufCFG_TxUE) != 0)
	   putline("\tEnable interrupt in Tx underrun\r\n");
	if ((val & PP_BufCFG_RxMiss) != 0)
	   putline("\tEnable interrupt on missed Rx packets\r\n");
	if ((val & PP_BufCFG_Rx128) != 0)
	   putline("\tEnable Rx interrupt after 128 bytes\r\n");
	if ((val & PP_BufCFG_TxCol) != 0)
	   putline("\tEnable int on Tx collision ctr overflow\r\n");
	if ((val & PP_BufCFG_Miss) != 0)
	   putline("\tEnable int on Rx miss ctr overflow\r\n");
	if ((val & PP_BufCFG_RxDest) != 0)
	   putline("\tEnable int on Rx dest addr match\r\n");

	val = *(ptr + PP_LineCTL);
	putline("Line control ox");
	putword(PP_LineCTL);
	putline(" Register ");
	putword(val & 0x3f);
	putline(" Value 0x");
	putword(val);
	putline("\r\n");
	if ((val & PP_LineCTL_Rx) != 0)
	   putline("\tEnable receiver\r\n");
	if ((val & PP_LineCTL_Tx) != 0)
	   putline("\tEnable transmitter\r\n");

	val = (*ptr + PP_RER);
	putline("Receive event 0x");
	putword(PP_RER);
	putline(" Register ");
	putword(val & 0x3f);
	putline(" Value 0x");
	putword(val);
	putline("\r\n");
	if ((val & PP_RER_IAHash) != 0)
	   putline("\tFrame hash match\r\n");
	if ((val & PP_RER_Dribble) != 0)
	   putline("\tFrame had 1-7 extra bits after last byte\r\n");
	if ((val & PP_RER_RxOK) != 0)
	   putline("\tFrame received with no errors\r\n");
	if ((val & PP_RER_Hashed) != 0)
	   putline("\tFrame address hashed OK\r\n");
	if ((val & PP_RER_IA) != 0)
	   putline("\tFrame address matched IA\r\n");
	if ((val & PP_RER_Broadcast) != 0)
	   putline("\tBroadcast frame\r\n");
	if ((val & PP_RER_CRC) != 0)
	   putline("\tFrame had CRC error\r\n");
	if ((val & PP_RER_RUNT) != 0)
	   putline("\tRunt frame\r\n");
	if ((val & PP_RER_EXTRA) != 0)
	   putline("\tFrame was too long\r\n");

	val = *(ptr + PP_TER);
	putline("Transmit event 0x");
	putword(PP_TER);
	putline(" Register ");
	putword(val & 0x3f);
	putline(" Value 0x");
	putword(val);
	putline("\r\n");
	if ((val & PP_TER_CRS) != 0)
	   putline("\tCarrier lost\r\n");
	if ((val & PP_TER_SQE) != 0)
	   putline("\tSignal Quality Error\r\n");
	if ((val & PP_TER_TxOK) != 0)
	   putline("\tPacket sent without error\r\n");
	if ((val & PP_TER_Late) != 0)
	   putline("\tOut of window\r\n");
	if ((val & PP_TER_Jabber) != 0)
	   putline("\tStuck transmit?\r\n");
	if ((val & PP_TER_NumCollisions) != 0)
	   putline("\tNumber of collisions\r\n");
	if ((val & PP_TER_16Collisions) != 0)
	   putline("\t> 16 collisions\r\n");

	val = *(ptr + PP_SelfCtl);
	putline("Chip control 0x");
	putword(PP_SelfCtl);
	putline(" Register ");
	putword(val & 0x3f);
	putline(" Value 0x");
	putword(val);
	putline("\r\n");
	if ((val & PP_SelfCtl_Reset) != 0)
	   putline("\tSelf-clearing reset\r\n");

	val = *(ptr + PP_BusCtl);
	putline("Bus control 0x");
	putword(PP_BusCtl);
	putline(" Register ");
	putword(val & 0x3f);
	putline(" Value 0x");
	putword(val);
	putline("\r\n");
	if ((val & PP_BusCtl_ResetRxDMA) != 0)
	   putline("\tReset receiver DMA engine\r\n");
	if ((val & PP_BusCtl_DMAextend) != 0)
	   putline("\t    0x0100\r\n");
	if ((val & PP_BusCtl_UseSA) != 0)
	   putline("\t        0x0200\r\n");
	if ((val & PP_BusCtl_MemoryE) != 0)
	   putline("\tEnable \"memory mode\"\r\n");
	if ((val & PP_BusCtl_DMAburst) != 0)
	   putline("\t     0x0800\r\n");
	if ((val & PP_BusCtl_IOCH_RDYE) != 0)
	   putline("\t    0x1000\r\n");
	if ((val & PP_BusCtl_RxDMAsize) != 0)
	   putline("\t    0x2000\r\n");
	if ((val & PP_BusCtl_EnableIRQ) != 0)
	   putline("\tEnable interrupts\r\n");

	val = *(ptr + PP_LineStat);
	putline("Line status 0x");
	putword(PP_LineStat);
	putline(" Register ");
	putword(val & 0x3f);
	putline(" Value 0x");
	putword(val);
	putline("\r\n");
	if ((val & PP_LineStat_LinkOK) != 0)
	   putline("\tLine is connected and working\r\n");
	if ((val & PP_LineStat_AUI) != 0)
	   putline("\tConnected via AUI\r\n");
	if ((val & PP_LineStat_10BT) != 0)
	   putline("\tConnected via twisted pair\r\n");
	if ((val & PP_LineStat_Polarity) != 0)
	   putline("\tLine polarity OK (10BT only)\r\n");
	if ((val & PP_LineStat_CRS) != 0)
	   putline("\tFrame being received\r\n");

	val = *(ptr + PP_SelfStat);
	putline("Chip status 0x");
	putword(PP_SelfStat);
	putline(" Register ");
	putword(val & 0x3f);
	putline(" Value 0x");
	putword(val);
	putline("\r\n");
	if ((val & PP_SelfStat_InitD) != 0)
	   putline("\tChip initialization complete\r\n");
	if ((val & PP_SelfStat_SIBSY) != 0)
	   putline("\tEEPROM is busy\r\n");
	if ((val & PP_SelfStat_EEPROM) != 0)
	   putline("\tEEPROM present\r\n");
	if ((val & PP_SelfStat_EEPROM_OK) != 0)
	   putline("\tEEPROM checks out\r\n");
	if ((val & PP_SelfStat_ELPresent) != 0)
	   putline("\tExternal address latch logic available\r\n");
	if ((val & PP_SelfStat_EEsize) != 0)
	   putline("\tSize of EEPROM\r\n");

	val = *(ptr + PP_BusStat);
	putline("Bus status 0x");
	putword(PP_BusStat);
	putline(" Register ");
	putword(val & 0x3f);
	putline(" Value 0x");
	putword(val);
	putline("\r\n");
	if ((val & PP_BusStat_TxBid) != 0)
	   putline("\tTx error\r\n");
	if ((val & PP_BusStat_TxRDY) != 0)
	   putline("\tReady for Tx data\r\n");

	putline("Logical address filter\r\n");
	putline("\t0x");
	for(i=6;i>=0;i-=2)
	{
		t = ptr + PP_LAF + i;
		val = *(ptr + PP_LAF + i);
		putword(val);
	}
	putline("\r\n");

	putline("MAC Address 0x"); 
	for(i = 4;i>= 0;i-=2)
	{
		t = ptr + PP_IA + i;
		val = *(ptr + PP_IA + i);
		putword(val);
	}
	putline("\r\n");
/*
	if ((val & PP_LAF       0x0150   Logical address filter (6 bytes)
	if ((val & PP_IA        0x0158   Individual address (MAC)
*/
}

#define MAC1 0x3000
#define MAC2 0xc6eb
#define MAC3 0x3049
#define IP 0xc0a8070a

unsigned char ip_addr[4];
unsigned char mac_addr[6];

void init_cs8900(unsigned short *cs8900)
{
	unsigned short stat;
	unsigned short *t;
	
	*(cs8900 + PP_LineCTL) = 0;		/* disable reception during MAC address settings */
	*(cs8900 + PP_IA) = MAC1;
	*(cs8900 + PP_IA + 2) = MAC2;
	*(cs8900 + PP_IA + 4) = MAC3;

	mac_addr[0] = MAC1 & 0xff;
	mac_addr[1] = (MAC1 >> 8) & 0xff;
	mac_addr[2] = MAC2 & 0xff;
	mac_addr[3] = (MAC2 >> 8) & 0xff;
	mac_addr[4] = MAC3 & 0xff;
	mac_addr[5] = (MAC3 >> 8) & 0xff;
	ip_addr[0] = (IP >> 24) & 0xff;
	ip_addr[1] = (IP >> 16) & 0xff;
	ip_addr[2] = (IP >> 8) & 0xff;
	ip_addr[3] = IP & 0xff;
	
	
	*(cs8900 + PP_LAF) = 0xffff;
	*(cs8900 + PP_LAF + 2) = 0xffff;
	*(cs8900 + PP_LAF + 4) = 0xffff;
	*(cs8900 + PP_LAF + 6) = 0xffff;

    *(cs8900 + PP_BusCtl) = PP_BusCtl_MemoryE;  /* Disable interrupts, memory mode */
    *(cs8900 + PP_IntReg) = PP_IntReg_IRQ0;  /* Only possibility */
    *(cs8900 + PP_RxCFG) = PP_RxCFG_RxOK | PP_RxCFG_CRC | 
                      PP_RxCFG_RUNT | PP_RxCFG_EXTRA;
    *(cs8900 + PP_RxCTL) = PP_RxCTL_RxOK | PP_RxCTL_Broadcast | PP_RxCTL_IA;
    *(cs8900 + PP_TxCFG) = PP_TxCFG_TxOK | PP_TxCFG_Collision | 
                      PP_TxCFG_CRS | PP_TxCFG_SQE | PP_TxCFG_Late | 
                      PP_TxCFG_Jabber | PP_TxCFG_16Collisions;
    *(cs8900 + PP_BufCFG) = PP_BufCFG_TxRDY | PP_BufCFG_TxUE | PP_BufCFG_RxMiss | 
                       PP_BufCFG_TxCol | PP_BufCFG_Miss | PP_BufCFG_SWI;
    *(cs8900 + PP_IntReg) = PP_IntReg_IRQ0;  /* Only possibility */
    *(cs8900 + PP_LineCTL) = PP_LineCTL_Rx | PP_LineCTL_Tx;
    /* Clear Interrupt Status Queue before enabling interrupts */
    do {
    	t = cs8900 + PP_ISQ;
        stat = *(cs8900 + PP_ISQ);
    }  while (stat != 0) ;
/*
    *(cs8900 + PP_BusCtl) = PP_BusCtl_EnableIRQ;
*/	
}

void poll_cs8900(unsigned short *cs8900)
{
	unsigned short event;
	unsigned short *t;
	
	t = cs8900 + PP_ISQ;
	event = *(cs8900 + PP_ISQ);
	while (event != 0)
	{
        switch (event & ISQ_EventMask) 
        {
        	case ISQ_RxEvent:
        					    rx_event(cs8900);
					            break;
	        case ISQ_TxEvent:
    					        tx_event(cs8900, event);
					            break;
		    case ISQ_BufEvent:
        					    buf_event(cs8900, event);
					            break;
	        case ISQ_RxMissEvent:
					            /* Receive miss counter has overflowed */
					            break;
	        case ISQ_TxColEvent:
					            /* Transmit collision counter has overflowed */
					            break;
	        default:
            					putline(__FUNCTION__);
            					putline (": Unknown event: ");
            					putword(event);
            					putline("\r\n");
					            break;
        }
        event = *(cs8900 + PP_ISQ);
  	}
	
}


int netin(unsigned short *cs8900,register struct epacket *packet,register int len)
{
	switch(net2hs(packet->ep_hdr.e_ptype))
	{
		case EP_ARP :	arp_in(cs8900,packet,len);
				break;
		case EP_RARP :	/*pack_up(packet,len);*/
				break;
		case EP_IP :	ip_in(cs8900,packet,len);
				break;
		default:	/* just drop packet */
				break;
	}
	return(TRUE);
}


int arp_in(unsigned short *cs8900,register struct epacket *packet,int len)
{
	short arop;
	register struct arppak *apacptr;

	apacptr = (struct arppak *)packet->ep_data;
	arop = net2hs(apacptr->ar_op);
	switch(arop)
	{
		case AR_REQ :	/* request - answer if for me */
				if (!memequ(ip_addr,apacptr->ar_tpa,IPLEN))
				{
					return(TRUE);
				}
				apacptr->ar_op = hs2net(AR_RPLY);
				memcpy(apacptr->ar_tpa,apacptr->ar_spa,IPLEN);
				memcpy(apacptr->ar_tha,packet->ep_hdr.e_src,EPADLEN);
				memcpy(packet->ep_hdr.e_dest,apacptr->ar_tha,EPADLEN);
				memcpy(apacptr->ar_sha,mac_addr,EPADLEN);
				memcpy(apacptr->ar_spa,ip_addr,IPLEN);
				memcpy(packet->ep_hdr.e_src,mac_addr,EPADLEN);
				tx_packet(cs8900,(unsigned short *)packet,EMINPAK);
				return(TRUE);
		case AR_RPLY:	/* reply - awaken requestor if any */
/*				pack_up(packet,len);*/
				return(TRUE);
		default:
				return(FALSE);
	}
}


int ip_in(unsigned short *cs8900,register struct epacket *packet,int len)
{
	register struct ip *ipptr;

	ipptr = (struct ip *)packet->ep_data;
	if ((ipptr->i_cksum != 0) && (cksum((unsigned char *)ipptr,IPHLEN) != 0))
	{
		return(FALSE);
	}
        switch(ipptr->i_proto)
        {
                case IPRO_ICMP: /* ICMP : pass to icmp input routine */
                                return(icmp_in(cs8900,packet));
                case IPRO_UDP:  /* UDP: demultiplex based on UDP "port" */
		case IPRO_TCP:  /* TCP : demux based on TCP port and type */
/*				pack_up(packet,len);*/
                default:        break;
        }
        return(TRUE);
}


int icmp_in(unsigned short *cs8900,register struct epacket *packet)
{
	register struct ip *ipptr;
	register struct icmp *icmpptr;
	int len;

	ipptr = (struct ip *)packet->ep_data;
	icmpptr = (struct icmp *)ipptr->i_data;
	switch(icmpptr->ic_typ)
	{
		case ICRQECH :
				memcpy(ipptr->i_dest,ipptr->i_src,IPLEN);
				memcpy(ipptr->i_src,ip_addr,IPLEN);
				len = net2hs(ipptr->i_paclen) - IPHLEN;
				if (isodd(len))
				{
					ipptr->i_data[len++] = 0;
				}
				icmpptr->ic_typ = (char)ICRPECH;
				icmpptr->ic_cksum = 0;
				icmpptr->ic_cksum = cksum((unsigned char *)icmpptr,len);
				memcpy(packet->ep_hdr.e_dest,packet->ep_hdr.e_src,EPADLEN);
				memcpy(packet->ep_hdr.e_src,mac_addr,EPADLEN);
				ipptr->i_cksum = 0;
				ipptr->i_cksum = cksum((unsigned char *)ipptr,IPHLEN);
				tx_packet(cs8900,(unsigned short *)packet,EHLEN + IPHLEN + len);
				break;
		default:	/*pack_up(packet,len);*/
				break;
	}
	return(TRUE);
}

void rx_event(unsigned short *cs8900)
{
	unsigned short status,len;
	unsigned short i,*t;
	
	t = cs8900 + RX_STATUS;
	status = *(cs8900 + RX_STATUS);
	len = *(cs8900 + RX_LEN);
	for(i=0;i<=len;i++)
		rx_packet[i] = *(cs8900 + RX_BUFFER + (i * 2));
	netin(cs8900,(struct epacket *)rx_packet,len);		
}

void tx_packet(unsigned short *cs8900,unsigned short *pkt,int len)
{
	int i;
	unsigned short stat;
	
	*(cs8900 + PP_TxCMD) = PP_TxCmd_TxStart_Full;
	*(cs8900 + PP_TxLEN) = len;
	do 
	{
        stat = *(cs8900 + PP_BusStat);
    } while (!(stat & PP_BusStat_TxRDY));

	for(i=0;i<len;i++)
		*(cs8900 + TX_BUFFER + (i * 2)) = *pkt++;
		
}

void tx_event(unsigned short *cs8900,unsigned short ev)
{
}

void buf_event(unsigned short *cs8900, unsigned short ev)
{
}



unsigned short  hs2net(unsigned short i)       /* host to network short */
{
	unsigned short t;

	t = i >> 8;
	i = i << 8;
	i += t;
	
	return(i);
}

unsigned short  net2hs(unsigned short i)       /* network to host short */
{
	unsigned short t;

	t = i >> 8;
	i = i << 8;
	i += t;
	return(i);
}

long    hl2net(long i)       /* host to network long */
{
	long t;

	t = (i & 255) << 24;
	t += ((i>> 8) & 255) << 16;
	t += ((i>> 16) & 255) << 8;
	t += (i>> 24) & 255;
	return(t);
}

long    net2hl(long i)       /* network to host long */
{
	long t;

	t = (i & 255) << 24;
	t += ((i>> 8) & 255) << 16;
	t += ((i>> 16) & 255) << 8;
	t += (i>> 24) & 255;
	return(t);
}


unsigned short cksum(register unsigned char buf[],register int nwords)
{
	register unsigned long sum;
	
	sum = 0L;
	while(nwords > 1)
	{
		sum += (unsigned long)(*buf++ << 8);
		sum += (unsigned long)*buf++;
		if (sum & 0x80000000L)
			sum = (sum & 0xffffL) + (sum >> 16);
		nwords -= 2;
	}
	if (nwords)
		sum += *buf;
	while (sum >> 16)
		sum = (sum & 0xffffL) + (sum >> 16);
	sum = ~sum;
	return(hs2net(sum & 0xffffL));
}

void memcpy(unsigned char *dest,unsigned char *src,int len)
{
	int i;
	
	for(i=0;i<len;i++)
		*dest++ = *src++;
}

int memequ(unsigned char *dest,unsigned char *src,int len)
{
	int i;
	
	for(i=0;i<len;i++)
		if (*dest++ != *src++)
			return(FALSE);
	return(TRUE);
}
